@@ -1,5 +1,8 @@
+from google.cloud.bigquery import RangePartitioning, PartitionRange, TimePartitioning
+
 from skt.ye import get_spark
 from google.cloud.bigquery.job import QueryJobConfig
+from google.cloud.exceptions import NotFound
 
 
 def _bq_cell_magic(line, query):
@@ -90,6 +93,14 @@ def get_bigquery_client():
     return client
 
 
+def bq_table_exists(table):
+    try:
+        get_bigquery_client().get_table(table)
+    except NotFound:
+        return False
+    return True
+
+
 def _get_partition_filter(dataset, table_name, partition):
     table = get_bigquery_client().get_table(f"{dataset}.{table_name}")
     if "timePartitioning" in table._properties:
@@ -104,6 +115,7 @@ def _get_partition_filter(dataset, table_name, partition):
 def bq_to_pandas(query, project_id="sktaic-datahub"):
     import pandas as pd
 
+    set_gcp_credentials()
     configuration = {"query": {"useQueryCache": True}}
     return pd.read_gbq(
         query=query, project_id=project_id, dialect="standard", use_bqstorage_api=True, configuration=configuration
@@ -172,7 +184,7 @@ def _df_to_bq_table(df, dataset, table_name, partition=None, mode="overwrite"):
     table = f"{dataset}.{table_name}${partition}" if partition else f"{dataset}.{table_name}"
     df.write.format("bigquery").option("project", "sktaic-datahub").option(
         "credentials", base64.b64encode(key.encode()).decode()
-    ).option("table", table).option("temporaryGcsBucket", "mnoai-us").save(mode=mode)
+    ).option("table", table).option("temporaryGcsBucket", "temp-seoul-7d").save(mode=mode)
 
 
 def df_to_bq_table(df, dataset, table_name, partition=None, mode="overwrite"):
@@ -226,3 +238,73 @@ def bq_to_df(query, spark_session=None):
     job.result()
 
     return _bq_table_to_df(temp_dataset, temp_table_name, "*", spark_session=spark_session)
+
+
+def load_query_result_to_table(dest_table, query, part_col_name=None, clustering_fields=None):
+    bq_client = get_bigquery_client()
+    qjc = None
+    print(query)
+    if bq_table_exists(dest_table):
+        table = bq_client.get_table(dest_table)
+        qjc = QueryJobConfig(
+            destination=dest_table,
+            write_disposition="WRITE_TRUNCATE",
+            create_disposition="CREATE_IF_NEEDED",
+            time_partitioning=table.time_partitioning,
+            range_partitioning=table.range_partitioning,
+            clustering_fields=table.clustering_fields,
+        )
+        job = bq_client.query(query, job_config=qjc)
+        job.result()
+
+    else:
+        import time
+
+        temp_table_name = f"load_query_result_to_table__{str(int(time.time()))}"
+        bq_client.query(f"CREATE OR REPLACE TABLE temp_1d.{temp_table_name} AS {query}").result()
+        if part_col_name:
+            schema = bq_client.get_table(f"temp_1d.{temp_table_name}").schema
+            partition_type = [f for f in schema if f.name.lower() == part_col_name.lower()][0].field_type
+            if partition_type == "DATE":
+                qjc = QueryJobConfig(
+                    destination=dest_table,
+                    write_disposition="WRITE_TRUNCATE",
+                    create_disposition="CREATE_IF_NEEDED",
+                    time_partitioning=TimePartitioning(field=part_col_name),
+                    clustering_fields=clustering_fields,
+                )
+            elif partition_type == "INTEGER":
+                qjc = QueryJobConfig(
+                    destination=dest_table,
+                    write_disposition="WRITE_TRUNCATE",
+                    create_disposition="CREATE_IF_NEEDED",
+                    range_partitioning=RangePartitioning(
+                        PartitionRange(start=200001, end=209912, interval=1), field=part_col_name
+                    ),
+                    clustering_fields=clustering_fields,
+                )
+            else:
+                print(partition_type)
+                raise Exception(f"Partition column[{part_col_name}] is neither DATE or INTEGER type.")
+        job = bq_client.query(f"SELECT * FROM temp_1d.{temp_table_name}", job_config=qjc)
+        job.result()
+
+
+def get_max_part(table_name):
+    from datetime import datetime
+
+    bq_client = get_bigquery_client()
+    parts = filter(lambda x: x != "__NULL__", get_bigquery_client().list_partitions(table_name))
+
+    if not parts:
+        raise Exception("Max partition value is invalid or null.")
+
+    max_part_value = max(parts)
+
+    table = bq_client.get_table(table_name)
+    if table.time_partitioning:
+        return datetime.strptime(max_part_value, "%Y%m%d").strftime("%Y-%m-%d")
+    elif table.range_partitioning:
+        return int(max_part_value)
+    else:
+        raise Exception("Partition column is neither DATE or INTEGER type.")
